@@ -1,4 +1,4 @@
-/* A Bison parser, made by GNU Bison 3.6.2.  */
+/* A Bison parser, made by GNU Bison 3.6.91.  */
 
 /* Bison interface for Yacc-like parsers in C
 
@@ -57,7 +57,7 @@ extern int gram_debug;
 
   #include "symlist.h"
   #include "symtab.h"
-#line 245 "src/parse-gram.y"
+#line 270 "src/parse-gram.y"
 
   typedef enum
   {
@@ -66,7 +66,7 @@ extern int gram_debug;
     param_parse  = 1 << 1,
     param_both   = param_lex | param_parse
   } param_type;
-#line 710 "src/parse-gram.y"
+#line 731 "src/parse-gram.y"
 
   #include "muscle-tab.h"
   typedef struct
@@ -220,5 +220,14 @@ struct GRAM_LTYPE
 
 
 int gram_parse (void);
+/* "%code provides" blocks.  */
+#line 27 "src/parse-gram.y"
+
+  /* Initialize unquote.  */
+  void parser_init (void);
+  /* Deallocate storage for unquote.  */
+  void parser_free (void);
+
+#line 232 "src/parse-gram.h"
 
 #endif /* !YY_GRAM_SRC_PARSE_GRAM_H_INCLUDED  */
